@@ -9,7 +9,7 @@
 #ifndef __DEV_DS1620_H__
 #define __DEV_DS1620_H__
 
-#include "utils.h"
+#include "dynamips_common.h"
 
 struct ds1620_data {
    u_int state;
