@@ -20,6 +20,8 @@
 #include "dynamips.h"
 #include "memory.h"
 #include "device.h"
+#include "fs_mbr.h"
+#include "fs_fat.h"
 
 #define DEBUG_ACCESS  0
 #define DEBUG_ATA     0
@@ -54,30 +56,143 @@
 
 /* Card Information Structure */
 static m_uint8_t cis_table[] = {
-   0x01, 0x03, 0xd9, 0x01, 0xff, 0x1c, 0x04, 0x03,
-   0xd9, 0x01, 0xff, 0x18, 0x02, 0xdf, 0x01, 0x20,
-   0x04, 0x34, 0x12, 0x00, 0x02, 0x15, 0x2b, 0x04,
-   0x01, 0x44, 0x79, 0x6e, 0x61, 0x6d, 0x69, 0x70,
-   0x73, 0x20, 0x41, 0x54, 0x41, 0x20, 0x46, 0x6c,
-   0x61, 0x73, 0x68, 0x20, 0x43, 0x61, 0x72, 0x64,
-   0x20, 0x20, 0x00, 0x44, 0x59, 0x4e, 0x41, 0x30,
-   0x20, 0x20, 0x00, 0x44, 0x59, 0x4e, 0x41, 0x30,
-   0x00, 0xff, 0x21, 0x02, 0x04, 0x01, 0x22, 0x02,
-   0x01, 0x01, 0x22, 0x03, 0x02, 0x04, 0x5f, 0x1a,
-   0x05, 0x01, 0x03, 0x00, 0x02, 0x0f, 0x1b, 0x0b,
-   0xc0, 0x40, 0xa1, 0x27, 0x55, 0x4d, 0x5d, 0x75,
-   0x08, 0x00, 0x21, 0x1b, 0x06, 0x00, 0x01, 0x21,
-   0xb5, 0x1e, 0x4d, 0x1b, 0x0d, 0xc1, 0x41, 0x99,
-   0x27, 0x55, 0x4d, 0x5d, 0x75, 0x64, 0xf0, 0xff,
-   0xff, 0x21, 0x1b, 0x06, 0x01, 0x01, 0x21, 0xb5,
-   0x1e, 0x4d, 0x1b, 0x12, 0xc2, 0x41, 0x99, 0x27,
-   0x55, 0x4d, 0x5d, 0x75, 0xea, 0x61, 0xf0, 0x01,
-   0x07, 0xf6, 0x03, 0x01, 0xee, 0x21, 0x1b, 0x06,
-   0x02, 0x01, 0x21, 0xb5, 0x1e, 0x4d, 0x1b, 0x12,
-   0xc3, 0x41, 0x99, 0x27, 0x55, 0x4d, 0x5d, 0x75,
-   0xea, 0x61, 0x70, 0x01, 0x07, 0x76, 0x03, 0x01,
-   0xee, 0x21, 0x1b, 0x06, 0x03, 0x01, 0x21, 0xb5,
-   0x1e, 0x4d, 0x14, 0x00,
+   /* CISTPL_DEVICE */
+   0x01, 0x03,
+      0xd9, /* DSPEED_250NS | WPS | DTYPE_FUNCSPEC */
+      0x01, /* 2 KBytes(Units)/64 KBytes(Max Size) */
+      0xff, /* 0xFF */
+   /* CISTPL_DEVICE_OC */
+   0x1c, 0x04,
+      0x03, /* MWAIT | 3.3 volt VCC operation */
+      0xd9, /* DSPEED_250NS | WPS | DTYPE_FUNCSPEC */
+      0x01, /* 2 KBytes(Units)/64 KBytes(Max Size) */
+      0xff, /* 0xFF */
+   /* CISTPL_JEDEC_C */
+   0x18, 0x02,
+      0xdf, /* PCMCIA */
+      0x01, /* 0x01 */
+   /* CISTPL_MANFID */
+   0x20, 0x04,
+      0x34, 0x12, /* 0x1234 ??? */
+      0x00, 0x02, /* 0x0200 */
+   /* CISTPL_VERS_1 */
+   0x15, 0x2b,
+      0x04, 0x01, /* PCMCIA 2.0/2.1 / JEIDA 4.1/4.2 */
+      0x44, 0x79, 0x6e, 0x61, 0x6d, 0x69, 0x70, 0x73,
+      0x20, 0x41, 0x54, 0x41, 0x20, 0x46, 0x6c, 0x61,
+      0x73, 0x68, 0x20, 0x43, 0x61, 0x72, 0x64, 0x20,
+      0x20, 0x00, /* "Dynamips ATA Flash Card  " */
+      0x44, 0x59, 0x4e, 0x41, 0x30, 0x20, 0x20, 0x00, /* "DYNA0  " */
+      0x44, 0x59, 0x4e, 0x41, 0x30, 0x00, /* "DYNA0" */
+      0xff, /* 0xFF */
+   /* CISTPL_FUNCID */
+   0x21, 0x02,
+      0x04, /* Fixed Disk */
+      0x01, /* Power-On Self Test */
+   /* CISTPL_FUNCE */
+   0x22, 0x02,
+      0x01, /* Disk Device Interface tuple */
+      0x01, /* PC Card-ATA Interface */
+   /* CISTPL_FUNCE:  */
+   0x22, 0x03,
+      0x02, /* Basic PC Card ATA Interface tuple */
+      0x04, /* S(Silicon Device) */
+      0x5f, /* P0(Sleep) | P1(Standy) | P2(Idle) | P3(Auto) | N(3F7/377 Register Inhibit Available) | I(IOIS16# on Twin Card) */
+   /* CISTPL_CONFIG */
+   0x1a, 0x05,
+      0x01, /* TPCC_RASZ=2, TPCC_RMSZ=1, TPCC_RFSZ=0 */
+      0x03, /* TPCC_LAST=3 */
+      0x00, 0x02, /* TPCC_RADR(Configuration Register Base Address)=0x0200 */
+      0x0f, /* TPCC_RMSK(Configuration Register Presence Mask Field)=0x0F */
+   /* CISTPL_CFTABLE_ENTRY */
+   0x1b, 0x0b,
+      0xc0, /* Index=0 | Default | Interface */
+      0x40, /* Memory | READY Active */
+      0xa1, /* VCC power-description-structure only | Single 2-byte length specified | Misc */
+      0x27, /* Nom V | Min V | Max V | Peak I */
+      0x55, /* Nom V=5V */
+      0x4d, /* Min V=4.5V */
+      0x5d, /* Max V=5V */
+      0x75, /* Peak I=80mA */
+      0x08, 0x00, /* Card Address=0 | Host Address=0x0008 * 256 bytes */
+      0x21, /* Max Twin Cards=1 | Power Down */
+   /* CISTPL_CFTABLE_ENTRY */
+   0x1b, 0x06,
+      0x00, /* Index=0 */
+      0x01, /* VCC power-description-structure only */
+      0x21, /* Nom V | Peak I */
+      0xb5, 0x1e, /* Nom V=3.30V */
+      0x4d, /* Peak I=45mA */
+   /* CISTPL_CFTABLE_ENTRY */
+   0x1b, 0x0d,
+      0xc1, /* Index=1 | Default | Interface */
+      0x41, /* I/O and Memory | READY Active */
+      0x99, /* VCC power-description-structure only | IO Space | IRQ | Misc */
+      0x27, /* Nom V | Min V | Max V | Peak I */
+      0x55, /* Nom V=5V */
+      0x4d, /* Min V=4.5V */
+      0x5d, /* Max V=5V */
+      0x75, /* Peak I=80mA */
+      0x64, /* IOAddrLines=4 | All registers are accessible by both 8-bit or 16-bit accesses */
+      0xf0, 0xff, 0xff, /* Mask | Level | Pulse | Share | IRQ0..IRQ15 */
+      0x21, /* Max Twin Cards=1 | Power Down */
+   /* CISTPL_CFTABLE_ENTRY */
+   0x1b, 0x06,
+      0x01, /* Index=1 */
+      0x01, /* VCC power-description-structure only */
+      0x21, /* Nom V | Peak I */
+      0xb5, 0x1e, /* Nom V=3.30V */
+      0x4d, /* Peak I=45mA */
+   /* CISTPL_CFTABLE_ENTRY */
+   0x1b, 0x12,
+      0xc2, /* Index=2 | Default | Interface */
+      0x41, /* I/O and Memory | READY Active */
+      0x99, /* VCC power-description-structure only | IO Space | IRQ | Misc */
+      0x27, /* Nom V | Min V | Max V | Peak I */
+      0x55, /* Nom V=5V */
+      0x4d, /* Min V=4.5V */
+      0x5d, /* Max V=5V */
+      0x75, /* Peak I=80mA */
+      0xea, /* IOAddrLines=10 | All registers are accessible by both 8-bit or 16-bit accesses | Range */
+      0x61, /* Number of I/O Address Ranges=2 | Size of Address=2 | Size of Length=1 */
+      0xf0, 0x01, 0x07, /* Address=0x1F0, Length=8 */
+      0xf6, 0x03, 0x01, /* Address=0x3F6, Length=2 */
+      0xee, /* IRQ14 | Level | Pulse | Share */
+      0x21, /* Max Twin Cards=1 | Power Down */
+   /* CISTPL_CFTABLE_ENTRY */
+   0x1b, 0x06,
+      0x02, /* Index=2 */
+      0x01, /* VCC power-description-structure only */
+      0x21, /* Nom V | Peak I */
+      0xb5, 0x1e, /* Nom V=3.30V */
+      0x4d, /* Peak I=45mA */
+   /* CISTPL_CFTABLE_ENTRY */
+   0x1b, 0x12,
+      0xc3, /* Index=3 | Default | Interface */
+      0x41, /* I/O and Memory | READY Active */
+      0x99, /* VCC power-description-structure only | IO Space | IRQ | Misc */
+      0x27, /* Nom V | Min V | Max V | Peak I */
+      0x55, /* Nom V=5V */
+      0x4d, /* Min V=4.5V */
+      0x5d, /* Max V=5V */
+      0x75, /* Peak I=80mA */
+      0xea, /* IOAddrLines=10 | All registers are accessible by both 8-bit or 16-bit accesses | Range */
+      0x61, /* Number of I/O Address Ranges=2 | Size of Address=2 | Size of Length=1 */
+      0x70, 0x01, 0x07, /* Address=0x170, Length=8 */
+      0x76, 0x03, 0x01, /* Address=0x376, Length=2 */
+      0xee, /* IRQ14 | Level | Pulse | Share */
+      0x21, /* Max Twin Cards=1 | Power Down */
+   /* CISTPL_CFTABLE_ENTRY */
+   0x1b, 0x06,
+      0x03, /* Index=3 */
+      0x01, /* VCC power-description-structure only */
+      0x21, /* Nom V | Peak I */
+      0xb5, 0x1e, /* Nom V=3.30V */
+      0x4d, /* Peak I=45mA */
+   /* CISTPL_NO_LINK */
+   0x14, 0x00,
+   /* CISTPL_END */
+   0xff,
 };
 
 /* PCMCIA private data */
@@ -123,14 +238,65 @@ static inline m_uint32_t chs_to_lba(struct pcmcia_disk_data *d,
    return((((cyl * d->nr_heads) + head) * d->sects_per_track) + sect - 1);
 }
 
+/* Convert a LBA reference to a CHS reference */
+static inline void lba_to_chs(struct pcmcia_disk_data *d,m_uint32_t lba,
+                              u_int *cyl,u_int *head,u_int *sect)
+{
+   *cyl = lba / (d->sects_per_track * d->nr_heads);
+   *head = (lba / d->sects_per_track) % d->nr_heads;
+   *sect = (lba % d->sects_per_track) + 1;
+}
+
+/* Format disk with a single FAT16 partition */
+static int disk_format(struct pcmcia_disk_data *d)
+{
+   struct mbr_data mbr;
+   struct mbr_partition *part;
+   u_int cyl=0, head=1, sect=1;
+
+   /* Master Boot Record */
+   memset(&mbr,0,sizeof(mbr));
+   mbr.signature[0] = MBR_SIGNATURE_0;
+   mbr.signature[1] = MBR_SIGNATURE_1;
+   part = &mbr.partition[0];
+   part->bootable = 0;
+   part->type = MBR_PARTITION_TYPE_FAT16;
+   part->lba = chs_to_lba(d, 0, 1, 1);
+   part->nr_sectors = d->nr_heads * d->nr_cylinders * d->sects_per_track - part->lba;
+   lba_to_chs(d, part->lba + part->nr_sectors - 1, &cyl, &head, &sect);
+   mbr_set_chs(part->first_chs, 0, 1, 1);
+   mbr_set_chs(part->last_chs, cyl, head, sect);
+
+   if (mbr_write_fd(d->fd, &mbr)<0) {
+      return(-1);
+   }
+
+   /* FAT16 partition */
+   if (fs_fat_format16(d->fd, part->lba, part->nr_sectors, d->sects_per_track,
+                       d->nr_heads, d->vm_obj.name)) {
+      return(-1);
+   }
+
+   return(0);
+}
+
 /* Create the virtual disk */
 static int disk_create(struct pcmcia_disk_data *d)
 {
    off_t disk_len;
 
-   if ((d->fd = open(d->filename,O_CREAT|O_RDWR,0600)) < 0) {
-      perror("disk_create: open");
-      return(-1);
+   if ((d->fd = open(d->filename,O_CREAT|O_EXCL|O_RDWR,0600)) < 0) {
+      /* already exists? */
+      if ((d->fd = open(d->filename,O_CREAT|O_RDWR,0600)) < 0) {
+         perror("disk_create: open");
+         return(-1);
+      }
+   }
+   else {
+      /* new disk */
+      if (disk_format(d)) {
+         return(-1);
+      }
    }
 
    disk_len = d->nr_heads * d->nr_cylinders * d->sects_per_track * SECTOR_SIZE;
@@ -196,8 +362,8 @@ static void ata_identify_device(struct pcmcia_disk_data *d)
    memset(p,0x00,SECTOR_SIZE);
 
    /* Word 0: General Configuration */
-   p[0] = 0x8a;
-   p[1] = 0x84;
+   p[0] = 0x8a; /* Not MFM encoded | Hard sectored | Removable cartridge drive */
+   p[1] = 0x84; /* Disk transfer rate !<= 10Mbs | Non-rotating disk drive */
 
    /* Word 1: Default number of cylinders */
    p[2] = d->nr_cylinders & 0xFF;
@@ -469,14 +635,14 @@ void *dev_pcmcia_disk_access_1(cpu_gen_t *cpu,struct vdevice *dev,
    if (op_type == MTS_READ) {
       cpu_log(cpu,d->dev.name,
               "reading offset 0x%5.5x at pc=0x%llx (size=%u)\n",
-              offset,cpu->pc,op_size);
+              offset,cpu_get_pc(cpu),op_size);
    } else {
       cpu_log(cpu,d->dev.name,
               "writing offset 0x%5.5x, data=0x%llx at pc=0x%llx (size=%u)\n",
-              offset,*data,cpu->pc,op_size);
+              offset,*data,cpu_get_pc(cpu),op_size);
    }
 #endif
-      
+
    switch(offset) {
       case 0x02:   /* Sector Count + Sector no */
          if (op_type == MTS_READ) {
@@ -503,10 +669,10 @@ void *dev_pcmcia_disk_access_1(cpu_gen_t *cpu,struct vdevice *dev,
             d->ata_cmd = *data >> 8;
             d->head = *data & 0xFF;
             ata_handle_cmd(d);
-         }            
+         }
          break;
 
-      case 0x08:
+      case 0x08:   /* Data */
          if (op_type == MTS_READ) {
             *data =  d->data_buffer[(d->data_pos << 1)];
             *data += d->data_buffer[(d->data_pos << 1)+1] << 8;
@@ -520,13 +686,13 @@ void *dev_pcmcia_disk_access_1(cpu_gen_t *cpu,struct vdevice *dev,
          /* Buffer complete: call the callback function */
          if (d->data_pos == (SECTOR_SIZE/2)) {
             d->data_pos = 0;
-               
+
             if (d->ata_cmd_callback)
                d->ata_cmd_callback(d);
          }
          break;
 
-      case 0x0E:
+      case 0x0E:   /* Status/Drive Control + Drive Address */
          break;
    }
 
