@@ -15,8 +15,8 @@
 #define JIT_SUPPORT 0
 
 /* Wrappers to x86-codegen functions */
-#define ppc32_jit_tcb_set_patch(a,b)
-#define ppc32_jit_tcb_set_jump(a,b)
+#define ppc32_jit_tcb_set_patch(a,b) (void)(a); (void)(b)
+#define ppc32_jit_tcb_set_jump(a,b)  (void)(a); (void)(b)
 
 /* PPC instruction array */
 extern struct ppc32_insn_tag ppc32_insn_tags[];
