@@ -6,23 +6,12 @@
 #ifndef __UTILS_H__
 #define __UTILS_H__
 
-#include <stdarg.h>
-#include <sys/types.h>
+#include "dynamips_common.h"
+
 #include <sys/time.h>
-#include <time.h>
 #include <netinet/in.h>
-#include <pthread.h>
 #include <signal.h>
 
-/* True/False definitions */
-#ifndef FALSE
-#define FALSE 0
-#endif
-
-#ifndef TRUE
-#define TRUE  1
-#endif
-
 /* Host CPU Types */
 #define CPU_x86    0
 #define CPU_amd64  1
@@ -37,31 +26,6 @@
 #define JIT_HOST_NREG  0
 #endif
 
-/* Endianness */
-#define ARCH_BIG_ENDIAN     0x4321
-#define ARCH_LITTLE_ENDIAN  0x1234
-
-#if defined(PPC) || defined(__powerpc__) || defined(__ppc__)
-#define ARCH_BYTE_ORDER ARCH_BIG_ENDIAN
-#elif defined(__sparc) || defined(__sparc__)
-#define ARCH_BYTE_ORDER ARCH_BIG_ENDIAN
-#elif defined(__alpha) || defined(__alpha__)
-#define ARCH_BYTE_ORDER ARCH_LITTLE_ENDIAN
-#elif defined(__i386) || defined(__i386__) || defined(i386)
-#define ARCH_BYTE_ORDER ARCH_LITTLE_ENDIAN
-#define ARCH_REGPARM_SUPPORTED  1
-#elif defined(__x86_64__)
-#define ARCH_BYTE_ORDER ARCH_LITTLE_ENDIAN
-#elif defined(__ia64__)
-#define ARCH_BYTE_ORDER ARCH_LITTLE_ENDIAN
-#elif defined(__arm__) || defined (__aarch64__)
-#define ARCH_BYTE_ORDER ARCH_LITTLE_ENDIAN
-#endif
-
-#ifndef ARCH_BYTE_ORDER
-#error Please define your architecture in utils.h!
-#endif
-
 /* Host to VM (big-endian) conversion functions */
 #if ARCH_BYTE_ORDER == ARCH_BIG_ENDIAN
 #define htovm16(x) (x)
@@ -81,63 +45,6 @@
 #define vmtoh64(x) (swap64(x))
 #endif
 
-/* Useful attributes for functions */
-#ifdef ARCH_REGPARM_SUPPORTED
-#define asmlinkage __attribute__((regparm(0)))
-#define fastcall   __attribute__((regparm(3)))
-#else
-#define asmlinkage
-#define fastcall
-#endif
-
-#ifndef __unused
-/* Function that is never used */
-#define __unused  __attribute__((unused))
-#endif
-
-#ifndef __maybe_unused
-/* Function that is referenced from excluded code (commented out or depends on preprocessor) */
-#define __maybe_unused  __attribute__((unused))
-#endif
-
-#ifndef UNUSED
-/* Variable that is never used (name is changed to get an error on use) */
-#define UNUSED(x)  UNUSED_ ## x __attribute__((unused))
-#endif
-
-#if __GNUC__ > 2
-#define forced_inline inline __attribute__((always_inline))
-#define no_inline __attribute__ ((noinline))
-#else
-#define forced_inline inline
-#define no_inline
-#endif
-
-#if __GNUC__ > 2
-/* http://kerneltrap.org/node/4705 */
-#define likely(x)    __builtin_expect(!!(x),1)
-#define unlikely(x)  __builtin_expect((x),0)
-#else
-#define likely(x)    (x)
-#define unlikely(x)  (x)
-#endif
-
-/* Common types */
-typedef unsigned char m_uint8_t;
-typedef signed char m_int8_t;
-
-typedef unsigned short m_uint16_t;
-typedef signed short m_int16_t;
-
-typedef unsigned int m_uint32_t;
-typedef signed int m_int32_t;
-
-typedef unsigned long long m_uint64_t;
-typedef signed long long m_int64_t;
-
-typedef unsigned long m_iptr_t;
-typedef m_uint64_t m_tmcnt_t;
-
 /* FD pool */
 #define FD_POOL_MAX  16
 
@@ -171,23 +78,6 @@ typedef m_uint32_t mips_insn_t;
 /* PowerPC instruction */
 typedef m_uint32_t ppc_insn_t;
 
-/* Max and min macro */
-#define m_max(a,b) (((a) > (b)) ? (a) : (b))
-#define m_min(a,b) (((a) < (b)) ? (a) : (b))
-
-/* A simple macro for adjusting pointers */
-#define PTR_ADJUST(type,ptr,size) (type)((char *)(ptr) + (size))
-
-/* Size of a field in a structure */
-#define SIZEOF(st,field) (sizeof(((st *)NULL)->field))
-
-/* Compute offset of a field in a structure */
-#define OFFSET(st,f)     ((long)&((st *)(NULL))->f)
-
-/* Stringify a constant */
-#define XSTRINGIFY(val)  #val
-#define STRINGIFY(val)   XSTRINGIFY(val)
-
 /* MMAP */
 #ifndef MAP_ANONYMOUS
 #define MAP_ANONYMOUS MAP_ANON
